/*
 * World Storage API
 *
 * API ensuring interoperability between an authoring tool and a World Storage service
 *
 * The version of the OpenAPI document: 0.0.3
 * 
 * Generated by: https://openapi-generator.tech
 */

using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Http;
using Swashbuckle.AspNetCore.Annotations;
using Swashbuckle.AspNetCore.SwaggerGen;
using Newtonsoft.Json;
using Org.OpenAPITools.Attributes;
using Org.OpenAPITools.Models;
using Org.OpenAPITools.Services;
using MongoDB.Driver;

namespace Org.OpenAPITools.Controllers
{
  /// <summary>
  /// 
  /// </summary>
  [ApiController]
  public class TrackablesApiControllerImpl : TrackablesApiController
  {

    private readonly TrackableService _trackableService;

    /// <summary>
    /// 
    /// </summary>
    public TrackablesApiControllerImpl(TrackableService trackableService)
    {
      _trackableService = trackableService;
    }


    /// <summary>
    /// Create a trackable
    /// </summary>
    /// <param name="trackable">the trackable to be added to the world storage</param>
    /// <response code="200">OK returns the UUID of the Trackable defined by the world storage.</response>
    /// <response code="201">Null response</response>
    /// <response code="0">unexpected error</response>
    [HttpPost]
    [Route("/trackables")]
    [Consumes("application/json", "application/xml")]
    [ValidateModelState]
    [SwaggerOperation("AddTrackable")]
    [SwaggerResponse(statusCode: 200, type: typeof(string), description: "OK returns the UUID of the Trackable defined by the world storage.")]
    [SwaggerResponse(statusCode: 0, type: typeof(Error), description: "unexpected error")]
    public override IActionResult AddTrackable([FromBody] Trackable trackable)
    {
      if (String.IsNullOrEmpty(trackable.UID.ToString()))
      {
        trackable.UID = Guid.NewGuid();
      }
      try
      {
        Trackable mytrackable = _trackableService.Create(trackable);
        return new ObjectResult(mytrackable);
      } catch (Exception e)
      {
        return StatusCode(400, e.Message);
      }
    }

    /// <summary>
    /// Deletes a trackable
    /// </summary>
    /// <param name="trackableId">trackable id to delete</param>
    /// <response code="200">OK</response>
    /// <response code="400">Invalid ID supplied</response>
    /// <response code="404">trackable not found</response>
    [HttpDelete]
    [Route("/trackables/{trackableId}")]
    [ValidateModelState]
    [SwaggerOperation("DeleteTrackable")]
    public override IActionResult DeleteTrackable([FromRoute(Name = "trackableId")][Required] string trackableId)
    {
      DeleteResult answer = _trackableService.Remove(Guid.Parse(trackableId));
      return (answer.IsAcknowledged && answer.DeletedCount > 0) ? new ObjectResult("ok") : new ObjectResult("failed");
    }

    /// <summary>
    /// Find a trackable by his ID
    /// </summary>
    /// <param name="trackableId">ID of the trackable to retrieve</param>
    /// <response code="200">successful operation</response>
    /// <response code="400">Invalid ID supplied</response>
    /// <response code="404">Trackable not found</response>
    [HttpGet]
    [Route("/trackables/{trackableId}")]
    [ValidateModelState]
    [SwaggerOperation("GetTrackableById")]
    [SwaggerResponse(statusCode: 200, type: typeof(Trackable), description: "successful operation")]
    public override IActionResult GetTrackableById([FromRoute(Name = "trackableId")][Required] string trackableId)
    {
      Trackable trackable = _trackableService.Get(Guid.Parse(trackableId));
      return (null != trackable) ? new ObjectResult(trackable) : new ObjectResult("not found"); 
    }

    /// <summary>
    /// returns the list of all trackables defined by the world storage.
    /// </summary>
    /// <response code="200">OK returns all the Trackables defined by the world storage.</response>
    /// <response code="201">Null response</response>
    /// <response code="0">unexpected error</response>
    [HttpGet]
    [Route("/trackables")]
    [ValidateModelState]
    [SwaggerOperation("GetTrackables")]
    [SwaggerResponse(statusCode: 200, type: typeof(List<Trackable>), description: "OK returns all the Trackables defined by the world storage.")]
    [SwaggerResponse(statusCode: 0, type: typeof(Error), description: "unexpected error")]
    public override IActionResult GetTrackables()
    {
      List<Trackable> trackablelist = _trackableService.Get();
      return new ObjectResult(trackablelist);
    }
  }
}
