﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Org.OpenAPITools.Models;
using MongoDB.Driver;

namespace Org.OpenAPITools.Services
{
  public class TrackableService
  {
    private readonly IMongoCollection<Trackable> _trackables;

    /// <summary>
    /// 
    /// </summary>
    public TrackableService(ITrackableDatabaseSettings settings)
    {
      var client = new MongoClient(settings.ConnectionString);
      var database = client.GetDatabase(settings.DatabaseName);

      _trackables = database.GetCollection<Trackable>(settings.CollectionName);
    }

    public List<Trackable> Get() =>
        _trackables.Find(trackable => true).ToList();

    public List<Trackable> Get(int limit) =>
        _trackables.Find(trackable => true).Limit(limit).ToList();

    public Trackable Get(Guid UID) =>
        _trackables.Find<Trackable>(trackable => trackable.UID == UID).FirstOrDefault();

    public Trackable Create(Trackable trackable)
    {
      _trackables.InsertOne(trackable);
      return trackable;
    }

    public ReplaceOneResult Update(Guid UID, Trackable trackableIn) =>
        _trackables.ReplaceOne(trackable => trackable.UID == UID, trackableIn);

    public DeleteResult Remove(Trackable trackableIn) =>
        _trackables.DeleteOne(trackable => trackable.UID == trackableIn.UID);

    public DeleteResult Remove(Guid UID) =>
        _trackables.DeleteOne(trackable => trackable.UID == UID);
  }
}

